﻿'=========================================================================================================
' Module:       ExifSupport
' Date:         16/10/2011
' Revised:      16/10/2011
' Author:       Tyron Harford (original http://channel9.msdn.com/coding4fun/articles/Photo-Screensaver)
'=========================================================================================================
Imports System
Imports System.Drawing
Imports System.Drawing.Imaging
Imports System.Text

Namespace Classes
    '''===================================================================================================
    ''' <summary>
    ''' Provides EXIF support for photo information.
    ''' </summary>
    '''===================================================================================================
    Public Class ExifSupport
        '''===============================================================================================
        ''' <summary>
        ''' Returns a long for the property.
        ''' </summary>
        ''' <param name="MyProperty">The property item to select a long for.</param>
        '''===============================================================================================
        Public Shared Function GetExifLong(ByVal MyProperty As PropertyItem) As Long
            If (MyProperty.Type <> 4) Then
                Throw New ArgumentException("Not an EXIF long value")
            End If
            Return CLng(Fix(((MyProperty.Value(0) Or (MyProperty.Value(1) << 8)) Or (MyProperty.Value(2) << &H10)) Or (MyProperty.Value(3) << &H18)))
        End Function
        '''===============================================================================================
        ''' <summary>
        ''' Returns a long for the bitmap and specified value.
        ''' </summary>
        ''' <param name="MyBitmap">The bitmap item to select a long for.</param>
        ''' <param name="ID">The ID value to select for.</param>
        ''' <param name="DefaultValue">The default value to return.</param>
        '''===============================================================================================
        Public Shared Function GetExifLong(ByVal MyBitmap As Bitmap, ByVal ID As Integer, ByVal DefaultValue As Long) As Long
            Try
                Return GetExifLong(MyBitmap.GetPropertyItem(ID))
            Catch
                Return DefaultValue
            End Try
        End Function
        '''===============================================================================================
        ''' <summary>
        ''' Returns a short for the property.
        ''' </summary>
        ''' <param name="MyProperty">The property item to select a short for.</param>
        '''===============================================================================================
        Public Shared Function GetExifShort(ByVal MyProperty As PropertyItem) As Short
            If (MyProperty.Type <> 3) Then
                Throw New ArgumentException("Not an EXIF short value")
            End If
            Return CShort(Fix(MyProperty.Value(0) Or (MyProperty.Value(1) << 8)))
        End Function
        '''===============================================================================================
        ''' <summary>
        ''' Returns a short for the bitmap and specified value.
        ''' </summary>
        ''' <param name="MyBitmap">The bitmap item to select a short for.</param>
        ''' <param name="ID">The ID value to select for.</param>
        ''' <param name="DefaultValue">The default value to return.</param>
        '''===============================================================================================
        Public Shared Function GetExifShort(ByVal MyBitmap As Bitmap, ByVal ID As Integer, ByVal DefaultValue As Short) As Short
            Try
                Return GetExifShort(MyBitmap.GetPropertyItem(ID))
            Catch
                Return DefaultValue
            End Try
        End Function
        '''===============================================================================================
        ''' <summary>
        ''' Returns a string for the property.
        ''' </summary>
        ''' <param name="MyProperty">The property item to select a string for.</param>
        '''===============================================================================================
        Public Shared Function GetExifString(ByVal MyProperty As PropertyItem) As String
            If (MyProperty.Type <> 2) Then
                Throw New ArgumentException("Not an EXIF string value")
            End If
            Dim aSCII As Encoding = Encoding.ASCII
            If MyProperty.Value Is Nothing Then
                Return Nothing
            End If
            Return aSCII.GetString(MyProperty.Value, 0, MyProperty.Value.Length - 1).Trim()
        End Function
        '''===============================================================================================
        ''' <summary>
        ''' Returns a short for the bitmap and specified value.
        ''' </summary>
        ''' <param name="MyBitmap">The bitmap item to select a short for.</param>
        ''' <param name="ID">The ID value to select for.</param>
        '''===============================================================================================
        Public Shared Function GetExifString(ByVal MyBitmap As Bitmap, ByVal ID As Integer) As String
            Try
                Return GetExifString(MyBitmap.GetPropertyItem(ID))
            Catch
                Return Nothing
            End Try
        End Function
        '''===============================================================================================
        ''' <summary>
        ''' Returns an object for the property.
        ''' </summary>
        ''' <param name="MyProperty">The property item to select a string for.</param>
        '''===============================================================================================
        Public Shared Function GetExifValue(ByVal MyProperty As PropertyItem) As Object
            Select Case MyProperty.Type
                Case 2
                    Return GetExifString(MyProperty)
                Case 3
                    Return GetExifShort(MyProperty)
                Case 4
                    Return GetExifLong(MyProperty)
            End Select
            Return Nothing
        End Function
        '''===============================================================================================
        ''' <summary>
        ''' Returns an object for the bitmap.
        ''' </summary>
        ''' <param name="MyBitmap">The bitmap item to select a short for.</param>
        ''' <param name="ID">The ID value to select for.</param>
        '''===============================================================================================
        Public Shared Function GetExifValue(ByVal MyBitmap As Bitmap, ByVal ID As Integer) As Object
            Try
                Return GetExifValue(MyBitmap.GetPropertyItem(ID))
            Catch
                Return Nothing
            End Try
        End Function
    End Class
End Namespace

