﻿'=========================================================================================================
' Module:       ConfigForm
' Date:         16/10/2011
' Revised:      16/10/2011
' Author:       Tyron Harford
'=========================================================================================================
Imports System
Imports System.ComponentModel
Imports System.Drawing
Imports System.Windows.Forms

Namespace Forms
    '''===================================================================================================
    ''' <summary>
    ''' Form used to configure the screensaver.
    ''' </summary>
    '''===================================================================================================
    Partial Public Class ConfigForm
        Inherits Form

        '''===============================================================================================
        ''' <summary>
        ''' Default constructor.
        ''' </summary>
        '''===============================================================================================
        Public Sub New()
            Me.InitializeComponent()

            ' retrieve the settings.
            Me.LoadSettings()
            Me.txtHelp.Text = My.Resources.Help
            Me.lblVersion.Text = My.Application.Info.Version.ToString
            Application.EnableVisualStyles()
        End Sub
        '''===============================================================================================
        ''' <summary>
        ''' Just exit.
        ''' </summary>
        '''===============================================================================================
        Private Sub btnCancel_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnCancel.Click
            MyBase.Close()
        End Sub
        '''===============================================================================================
        ''' <summary>
        ''' Reset the settings.
        ''' </summary>
        '''===============================================================================================
        Private Sub btnReset_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnReset.Click
            Dim msg As New System.Text.StringBuilder
            msg.Append("Are you sure you want to reset to default values (changes are instant)?")
            msg.AppendLine()
            msg.Append("(current saved directories and custom quotes will be retained)")
            msg.AppendLine()
            '
            If (MessageBox.Show(msg.ToString, "ViperWorks.Collage", MessageBoxButtons.YesNo) = vbYes) Then
                Dim tempDirs As Specialized.StringCollection
                Dim tempQuotes As Specialized.StringCollection
                '
                tempDirs = My.Settings.PhotoPaths
                tempQuotes = My.Settings.CustomQuotes
                '
                My.Settings.Reset()
                My.Settings.PhotoPaths = tempDirs
                My.Settings.CustomQuotes = tempQuotes
                My.Settings.Save()
                Me.LoadSettings()
            End If
        End Sub
        '''===============================================================================================
        ''' <summary>
        ''' Save the changes.
        ''' </summary>
        '''===============================================================================================
        Private Sub btnSave_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnSave.Click
            My.Settings.BarDrawFormat = Me.cbDisplay.Text
            My.Settings.CaptionFormat = Me.cbCaptionFormat.Text
            My.Settings.QuoteFormat = Me.cbQuote.Text
            My.Settings.AdditionalQuotes = Me.cbAdditionalTypes.Text
            My.Settings.Lowercase = Me.chbUseLowercase.Checked
            My.Settings.BoldCaptions = Me.chbBoldCaption.Checked
            My.Settings.ShowClock = Me.chbShowClock.Checked
            My.Settings.DimScreen = Me.chbDimScreen.Checked
            My.Settings.AllBlacksMode = Me.chbAllBlacks.Checked
            '
            My.Settings.DateTimeFormat = Me.txtDateTimeFormat.Text
            My.Settings.Interval = CInt(Fix(Me.nudInterval.Value))
            My.Settings.BarOpacity = CInt(Fix(Me.nudOpacity.Value))
            My.Settings.InfoInterval = CInt(Fix(Me.nudInfoInterval.Value))
            My.Settings.PercentQuotes = CInt(Fix(Me.nudPercentQuotes.Value))
            My.Settings.PercentWords = CInt(Fix(Me.nudPercentWords.Value))
            '
            My.Settings.SidebarPosition = Me.cbSidebarPosition.Text
            My.Settings.SidebarColor = Me.cbSidebarColor.Text
            My.Settings.SidebarDateFontColor = Me.cbSidebarDateColor.Text
            My.Settings.SidebarInfoFontColor = Me.cbSidebarInfoColor.Text
            My.Settings.PhotoCaptionColor = Me.cbPhotoCaptionColor.Text
            My.Settings.PhotoBackgroundColor = Me.cbPhotoBackColor.Text
            '
            If (My.Settings.PhotoPaths Is Nothing) Then
                My.Settings.PhotoPaths = New Specialized.StringCollection
            End If
            My.Settings.PhotoPaths.Clear()
            For Each PhotoDirectory As String In Me.txtDirectories.Lines
                If PhotoDirectory.Trim() <> "" Then
                    My.Settings.PhotoPaths.Add(PhotoDirectory)
                End If
            Next PhotoDirectory
            '
            If My.Settings.CustomQuotes Is Nothing Then
                My.Settings.CustomQuotes = New Specialized.StringCollection()
            End If
            My.Settings.CustomQuotes.Clear()
            For Each quote As String In Me.txtQuotes.Lines
                If quote.Trim() <> "" Then
                    My.Settings.CustomQuotes.Add(quote)
                End If
            Next quote
            '
            My.Settings.CaptionFont = Me.cbFontPhoto.Text
            My.Settings.SidebarFont = Me.cbFontSidebar.Text
            My.Settings.SidebarDateFont = Me.cbFontSidebarDate.Text

            My.Settings.CaptionSize = CInt(Me.nudFontPhotoSize.Value)
            My.Settings.SidebarSize = CInt(Me.nudFontSidebarSize.Value)
            My.Settings.SidebarDateSize = CInt(Me.nudFontSidebarDateSize.Value)
            '
            My.Settings.Save()
            Me.Close()
        End Sub
        '''===============================================================================================
        ''' <summary>
        ''' Give the user some feedback as to how many quotes will be used.
        ''' </summary>
        '''===============================================================================================
        Private Sub nudPercentQuotes_ValueChanged(ByVal sender As Object, ByVal e As EventArgs) Handles nudPercentQuotes.ValueChanged
            If nudPercentQuotes.Value = 100 Then
                Me.lblQuote.Text = "Percentage of Quotes (All quotes will be used):"
            Else
                Dim percent As Double = CInt(Fix(nudPercentQuotes.Value)) * 0.01
                Dim total As Integer = 0
                total = CInt(Fix(Program.Quotes.Count * percent))
                '
                Me.lblQuote.Text = String.Format("Percentage of Quotes ({0} quotes will be used):", total)
            End If
        End Sub
        '''===============================================================================================
        ''' <summary>
        ''' Give the user some feedback as to how many words will be used.
        ''' </summary>
        '''===============================================================================================
        Private Sub nudPercentWords_ValueChanged(ByVal sender As Object, ByVal e As EventArgs) Handles nudPercentWords.ValueChanged
            If nudPercentWords.Value = 100 Then
                Me.lblWord.Text = "Percentage of Word Definitions (All words will be used):"
            Else
                Dim percent As Double = CInt(Fix(nudPercentWords.Value)) * 0.01
                Dim total As Integer = 0
                total = CInt(Fix(Program.Words.Count * percent))
                '
                Me.lblWord.Text = String.Format("Percentage of Word Definitions ({0} words will be used):", total)
            End If
        End Sub
        '''===============================================================================================
        ''' <summary>
        ''' When the photo interval changes, change the info minimum value to match the photo
        ''' interval. This is done because the info interval uses the paint time of photos.
        ''' </summary>
        '''===============================================================================================
        Private Sub nudInterval_ValueChanged(ByVal sender As Object, ByVal e As EventArgs) Handles nudInterval.ValueChanged
            Me.nudInfoInterval.Minimum = nudInterval.Value
        End Sub
        '''===============================================================================================
        ''' <summary>
        ''' Sets the controls values to the settings.
        ''' </summary>
        '''===============================================================================================
        Private Sub LoadSettings()
            Me.cbDisplay.Text = My.Settings.BarDrawFormat
            Me.cbCaptionFormat.Text = My.Settings.CaptionFormat
            Me.cbQuote.Text = My.Settings.QuoteFormat
            Me.cbAdditionalTypes.Text = My.Settings.AdditionalQuotes
            Me.chbUseLowercase.Checked = My.Settings.Lowercase
            Me.chbBoldCaption.Checked = My.Settings.BoldCaptions
            Me.chbShowClock.Checked = My.Settings.ShowClock
            Me.chbDimScreen.Checked = My.Settings.DimScreen
            Me.chbAllBlacks.Checked = My.Settings.AllBlacksMode
            '
            Me.cbSidebarPosition.Text = My.Settings.SidebarPosition
            Me.txtDateTimeFormat.Text = My.Settings.DateTimeFormat
            Me.nudInterval.Value = My.Settings.Interval
            Me.nudOpacity.Value = My.Settings.BarOpacity
            Me.nudPercentQuotes.Value = My.Settings.PercentQuotes
            Me.nudPercentWords.Value = My.Settings.PercentWords
            Try
                Me.nudInfoInterval.Value = My.Settings.InfoInterval
            Catch e1 As Exception
            End Try
            '
            Try
                Me.txtDirectories.Text = ""
                If (My.Settings.PhotoPaths IsNot Nothing) Then
                    For Each PhotoDirectory As String In My.Settings.PhotoPaths
                        Me.txtDirectories.Text += PhotoDirectory & Environment.NewLine
                    Next PhotoDirectory
                    Me.txtDirectories.Text = Me.txtDirectories.Text.Trim()
                Else
                    Me.txtDirectories.Text = ""
                End If
            Catch ex As Exception
                Me.txtDirectories.Text = ""
            End Try
            '
            Try
                Me.txtQuotes.Text = ""
                If My.Settings.CustomQuotes IsNot Nothing Then
                    For Each quote As String In My.Settings.CustomQuotes
                        Me.txtQuotes.Text += quote & Environment.NewLine
                    Next quote
                    Me.txtQuotes.Text = Me.txtQuotes.Text.Trim()
                Else
                    Me.txtQuotes.Text = ""
                End If
            Catch ex As Exception
                Me.txtQuotes.Text = ""
            End Try

            ' fonts
            Dim fontFamilies() As FontFamily
            Dim installedFontCollection As New Drawing.Text.InstalledFontCollection()

            ' Get the array of FontFamily objects.
            fontFamilies = installedFontCollection.Families
            Me.cbFontPhoto.Items.Clear()
            Me.cbFontSidebar.Items.Clear()
            Me.cbFontSidebarDate.Items.Clear()
            For i As Integer = 0 To fontFamilies.Length - 1
                Me.cbFontPhoto.Items.Add(fontFamilies(i).Name)
                Me.cbFontSidebar.Items.Add(fontFamilies(i).Name)
                Me.cbFontSidebarDate.Items.Add(fontFamilies(i).Name)
            Next
            Me.cbFontPhoto.Text = My.Settings.CaptionFont
            Me.cbFontSidebar.Text = My.Settings.SidebarFont
            Me.cbFontSidebarDate.Text = My.Settings.SidebarDateFont
            '
            Me.nudFontPhotoSize.Value = My.Settings.CaptionSize
            Me.nudFontSidebarSize.Value = My.Settings.SidebarSize
            Me.nudFontSidebarDateSize.Value = My.Settings.SidebarDateSize

            ' get colors.
            Me.cbSidebarColor.Items.Clear()
            Me.cbSidebarDateColor.Items.Clear()
            Me.cbSidebarInfoColor.Items.Clear()
            Me.cbPhotoCaptionColor.Items.Clear()
            Me.cbPhotoBackColor.Items.Clear()
            '
            Me.cbSidebarColor.Items.AddRange(Me.GetColors())
            Me.cbSidebarDateColor.Items.AddRange(Me.GetColors())
            Me.cbSidebarInfoColor.Items.AddRange(Me.GetColors())
            Me.cbPhotoCaptionColor.Items.AddRange(Me.GetColors())
            Me.cbPhotoBackColor.Items.AddRange(Me.GetColors())
            '
            Me.cbSidebarColor.Text = My.Settings.SidebarColor
            Me.cbSidebarDateColor.Text = My.Settings.SidebarDateFontColor
            Me.cbSidebarInfoColor.Text = My.Settings.SidebarInfoFontColor
            Me.cbPhotoCaptionColor.Text = My.Settings.PhotoCaptionColor
            Me.cbPhotoBackColor.Text = My.Settings.PhotoBackgroundColor
        End Sub
        '''===============================================================================================
        ''' <summary>
        ''' Returns a list of known colors.
        ''' </summary>
        '''===============================================================================================
        Private Function GetColors() As String()
            Dim colors As New List(Of String)()
            Dim colorNames As String() = [Enum].GetNames(GetType(KnownColor))
            '
            For Each colorName As String In colorNames
                Dim knownColor As KnownColor
                knownColor = DirectCast([Enum].Parse(GetType(KnownColor), colorName), KnownColor)
                '
                If knownColor > knownColor.Transparent Then
                    colors.Add(colorName)
                End If
            Next
            '
            Return colors.ToArray
        End Function
    End Class
End Namespace

