﻿'=========================================================================================================
' Module:       PhotoInfo
' Date:         16/10/2011
' Revised:      16/10/2011
' Author:       Tyron Harford (original http://channel9.msdn.com/coding4fun/articles/Photo-Screensaver)
'=========================================================================================================
Imports System
Imports System.Diagnostics
Imports System.Drawing
Imports System.Drawing.Drawing2D
Imports System.Drawing.Imaging
Imports System.IO
Imports System.Text
Imports System.Xml
Imports System.Windows.Media.Imaging

Namespace Models
    '''===================================================================================================
    ''' <summary>
    ''' Represents a photo.
    ''' </summary>
    '''===================================================================================================
    Public Class PhotoInfo
        Private _Caption As String
        Private _MetaCaption As String
        Private _Filename As String
        Private _DateTaken As String
        Private _SourceBitmap As New WeakReference(Nothing)
        Private _WorkingBitmap As New WeakReference(Nothing)

#Region " Properties "
        '''===============================================================================================
        ''' <summary>
        ''' Gets or sets the photo caption.
        ''' </summary>
        '''===============================================================================================
        Public Property Caption() As String
            Get
                If Me._Caption Is Nothing Then
                    Me.GetPhotoCaption()
                End If
                Return Me._Caption
            End Get
            Set(ByVal value As String)
                Me._Caption = value
            End Set
        End Property
        '''===============================================================================================
        ''' <summary>
        ''' Gets or sets the filename of the photo.
        ''' </summary>
        '''===============================================================================================
        Public Property Filename() As String
            Get
                Return Me._Filename
            End Get
            Set(ByVal value As String)
                Me._Filename = value
            End Set
        End Property
        '''===============================================================================================
        ''' <summary>
        ''' Gets or sets the source bitmap.
        ''' </summary>
        '''===============================================================================================
        Public Property SourceBitmap() As Bitmap
            Get
                If (Not Me._SourceBitmap.IsAlive) Then
                    Try
                        Return Me.ReadImage()
                    Catch exception As Exception
                        If Debugger.IsAttached Then
                            Dim bitmap As New Bitmap(240, 240)
                            Me._SourceBitmap.Target = bitmap
                            Using graphics As Graphics = graphics.FromImage(CType(Me._SourceBitmap.Target, Bitmap))
                                graphics.Clear(Color.LightPink)
                                graphics.DrawString(Me._Filename, New Font("Times New Roman", 8.0F), Brushes.Black, CSng(0.0F), CSng(120.0F))
                                graphics.DrawString(exception.ToString(), New Font("Times New Roman", 8.0F), Brushes.Black, CSng(0.0F), CSng(130.0F))
                            End Using
                            Return bitmap
                        End If
                        Return Nothing
                    End Try
                End If
                Return CType(Me._SourceBitmap.Target, Bitmap)
            End Get
            Set(ByVal value As Bitmap)
                Me.ReplaceBitmapWeakReference(Me._SourceBitmap, value)
            End Set
        End Property
        '''===============================================================================================
        ''' <summary>
        ''' Gets or sets the working bitmap.
        ''' </summary>
        '''===============================================================================================
        Public Property WorkingBitmap() As Bitmap
            Get
                If (Not Me._WorkingBitmap.IsAlive) Then
                    Return Nothing
                End If
                Return CType(Me._WorkingBitmap.Target, Bitmap)
            End Get
            Set(ByVal value As Bitmap)
                Me.ReplaceBitmapWeakReference(Me._WorkingBitmap, value)
            End Set
        End Property
#End Region

        '''===============================================================================================
        ''' <summary>
        ''' Constructor.
        ''' </summary>
        ''' <param name="Filename">The physical file of the photo.</param>
        '''===============================================================================================
        Public Sub New(ByVal Filename As String)
            Me._Filename = Filename
        End Sub
        '''===============================================================================================
        ''' <summary>
        ''' Constrains the size of a photo.
        ''' </summary>
        '''===============================================================================================
        Public Shared Function ConstrainSize(ByVal SourcePhoto As Bitmap, ByVal Width As Integer, ByVal Height As Integer, ByVal CropToFit As Boolean) As Bitmap
            Dim num As Integer = SourcePhoto.Width
            Dim num2 As Integer = SourcePhoto.Height
            Dim x As Integer = 0
            Dim y As Integer = 0
            Dim num5 As Single = 0.0F
            Dim num6 As Single = 0.0F
            Dim num7 As Single = 0.0F
            num6 = (CSng(Width)) / (CSng(num))
            num7 = (CSng(Height)) / (CSng(num2))
            If (num7 > num6) AndAlso CropToFit Then
                num5 = num7
                x = CShort(Fix((Width - (num * num5)) / 2.0F))
            Else
                num5 = num6
                y = CShort(Fix((Height - (num2 * num5)) / 2.0F))
            End If
            Dim num8 As Integer = CInt(Fix(num * num5))
            Dim num9 As Integer = CInt(Fix(num2 * num5))
            Dim image As New Bitmap(Width, Height, PixelFormat.Format24bppRgb)
            Using graphics As Graphics = graphics.FromImage(image)
                SetDrawingQuality(graphics)
                graphics.Clear(SourcePhoto.GetPixel(0, 0))
                graphics.DrawImage(SourcePhoto, New Rectangle(x, y, num8, num9), New Rectangle(0, 0, num, num2), GraphicsUnit.Pixel)
            End Using
            Return image
        End Function
        '''===============================================================================================
        ''' <summary>
        ''' Gets the photo caption.
        ''' </summary>
        '''===============================================================================================
        Private Sub GetPhotoCaption()
            If String.IsNullOrEmpty(Me._Caption) Then
                Select Case My.Settings.CaptionFormat.ToLower()
                    Case "none"
                        Me._Caption = " "
                    Case "date"
                        Me._Caption = Me._DateTaken
                    Case "caption"
                        Me.SetCaption()
                    Case "caption + date"
                        If String.IsNullOrEmpty(Me._Caption) Then
                            Me.SetCaption()
                        End If
                        Me._Caption &= Environment.NewLine
                        If Me._Caption.Trim() = "" Then
                            Me._Caption = Me._Caption.Trim()
                        End If
                        Me._Caption &= Me._DateTaken
                End Select
            End If
        End Sub
        '''===============================================================================================
        ''' <summary>
        ''' Sets the caption using the filename if the metacaption is not available.
        ''' </summary>
        '''===============================================================================================
        Private Sub SetCaption()
            If String.IsNullOrEmpty(Me._MetaCaption) = False Then
                Me._Caption = Me._MetaCaption
            Else
                Dim startIndex As Integer = Me._Filename.LastIndexOf("\") + 1
                Dim num2 As Integer = Me._Filename.LastIndexOf(".")
                If ((startIndex > -1) AndAlso (startIndex < num2)) AndAlso (num2 > -1) Then
                    Me._Caption = Me._Filename.Substring(startIndex, num2 - startIndex)
                End If
            End If
        End Sub
        '''===============================================================================================
        ''' <summary>
        ''' Reads the photo in.
        ''' </summary>
        '''===============================================================================================
        Private Function ReadImage() As Bitmap
            Dim stream As FileStream = New FileInfo(Me._Filename).OpenRead()
            Dim length As Integer = CInt(Fix(stream.Length))
            Dim buffer(length - 1) As Byte
            stream.Read(buffer, 0, length)

            ' try and get the caption and date taken from the photo itself.
            Try
                stream.Position = 0
                Dim location As New Uri(Me._Filename)
                Dim metadata As BitmapMetadata
                Dim dateBuffer As DateTime
                Dim decoder As New JpegBitmapDecoder(stream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.Default)
                metadata = CType(decoder.Frames(0).Metadata, BitmapMetadata)
                dateBuffer = DateTime.Parse(metadata.DateTaken)
                Try
                    _DateTaken = dateBuffer.ToString(My.Settings.DateTimeFormat)
                Catch ex As Exception
                    _DateTaken = dateBuffer.ToString("d MMM yyyy")
                End Try
                _MetaCaption = metadata.Title
            Catch e1 As Exception
                _DateTaken = String.Empty
            End Try
            '
            stream.Close()
            Dim stream2 As New MemoryStream(buffer)
            Dim bitmap As Bitmap = CType(Image.FromStream(stream2), Bitmap)
            '
            stream2.Close()
            Me._SourceBitmap.Target = bitmap
            Return bitmap
        End Function
        '''===============================================================================================
        ''' <summary>
        ''' Replace references.
        ''' </summary>
        '''===============================================================================================
        Private Sub ReplaceBitmapWeakReference(ByVal weakRef As WeakReference, ByVal newBitmap As Bitmap)
            If weakRef.IsAlive Then
                CType(weakRef.Target, Bitmap).Dispose()
            End If
            weakRef.Target = newBitmap
        End Sub
        '''===============================================================================================
        ''' <summary>
        ''' Sets the drawing quality of the image.
        ''' </summary>
        '''===============================================================================================
        Public Shared Sub SetDrawingQuality(ByVal g As Graphics)
            g.InterpolationMode = InterpolationMode.HighQualityBicubic
            g.SmoothingMode = SmoothingMode.AntiAlias
            g.CompositingQuality = CompositingQuality.HighQuality
        End Sub
    End Class
End Namespace

