﻿'=========================================================================================================
' Module:       Program
' Date:         16/10/2011
' Revised:      16/10/2011
' Author:       Tyron Harford (original http://channel9.msdn.com/coding4fun/articles/Photo-Screensaver)
'=========================================================================================================
Imports System
Imports System.Windows.Forms
Imports System.Collections.Generic
Imports ViperWorks.Media.Collage.Models
Imports ViperWorks.Media.Collage.Classes
Imports ViperWorks.Media.Collage.Forms

Friend NotInheritable Class Program

#Region " Class Definition "
    Private Shared _Quotes As New List(Of DisplayItem)()
    Private Shared _CustomQuotes As New List(Of DisplayItem)()
    Private Shared _Words As New List(Of DisplayItem)()
    Private Shared _Events As New List(Of DisplayItem)()
    Private Shared _QuoteCeiling As Integer
    Private Shared _WordsCeiling As Integer
    Private Shared _MyRandom As New Random()
    Private Shared _RunningAsScreensaver As Boolean

    '''===================================================================================================
    ''' <summary>
    ''' If we are in screensaver mode.
    ''' </summary>
    '''===================================================================================================
    Public Shared ReadOnly Property RunningAsScreensaver() As Boolean
        Get
            Return _RunningAsScreensaver
        End Get
    End Property
    '''===================================================================================================
    ''' <summary>
    ''' Provides access to full quote library.
    ''' </summary>
    '''===================================================================================================
    Public Shared ReadOnly Property Quotes() As List(Of DisplayItem)
        Get
            Return _Quotes
        End Get
    End Property
    '''===================================================================================================
    ''' <summary>
    ''' Provides access to full custom quote library.
    ''' </summary>
    '''===================================================================================================
    Public Shared ReadOnly Property CustomQuotes() As List(Of DisplayItem)
        Get
            Return _CustomQuotes
        End Get
    End Property
    '''===================================================================================================
    ''' <summary>
    ''' Provides access to full words library.
    ''' </summary>
    '''===================================================================================================
    Public Shared ReadOnly Property Words() As List(Of DisplayItem)
        Get
            Return _Words
        End Get
    End Property
    '''===================================================================================================
    ''' <summary>
    ''' Provides access to full events library.
    ''' </summary>
    '''===================================================================================================
    Public Shared ReadOnly Property Events() As List(Of DisplayItem)
        Get
            Return _Events
        End Get
    End Property
    '''===================================================================================================
    ''' <summary>
    ''' Returns a random quote within the selected ceiling.
    ''' </summary>
    '''===================================================================================================
    Public Shared ReadOnly Property RandomQuote() As DisplayItem
        Get
            ' determine which quote place to use, custom or built in.
            ' if we have no custom quotes, force to use builtin.
            Dim bucket As Integer = _MyRandom.Next(0, 2)
            If CustomQuotes.Count = 0 Then
                bucket = 1
            End If

            ' if we use bucket 0, we are using custom quotes.
            If bucket = 0 Then
                Dim pos As Integer = _MyRandom.Next(0, CustomQuotes.Count)
                Return CustomQuotes(pos)
            Else
                ' if we are on anything else, use built in quotes.
                If Quotes.Count <= 0 Then
                    Return Nothing
                Else
                    Dim pos As Integer = _MyRandom.Next(0, _QuoteCeiling)
                    Return Quotes(pos)
                End If
            End If
        End Get
    End Property
    '''===================================================================================================
    ''' <summary>
    ''' Returns a random word within the selected ceiling.
    ''' </summary>
    '''===================================================================================================
    Public Shared ReadOnly Property RandomWord() As DisplayItem
        Get
            If Words.Count <= 0 Then
                Return Nothing
            Else
                Dim pos As Integer = _MyRandom.Next(0, _WordsCeiling)
                Return Words(pos)
            End If
        End Get
    End Property
    '''===================================================================================================
    ''' <summary>
    ''' Returns a random event that happens today.
    ''' </summary>
    '''===================================================================================================
    Public Shared ReadOnly Property RandomEvent() As DisplayItem
        Get
            If Events.Count <= 0 Then
                Return Nothing
            Else
                ' get a filtered list of events that happened on this day and month.
                Dim FilteredEvents As List(Of DisplayItem)
                FilteredEvents = Events.FindAll(
                    Function(dItem As DisplayItem) dItem.Day = DateTime.Now.Day AndAlso dItem.Month = DateTime.Now.Month)

                ' we have our list, get a random one in that list.
                Dim pos As Integer = _MyRandom.Next(0, FilteredEvents.Count)
                Return FilteredEvents(pos)
            End If
        End Get
    End Property
#End Region

#Region " Constructor "
    '''===================================================================================================
    ''' <summary>
    ''' Default constructor.
    ''' </summary>
    '''===================================================================================================
    Private Sub New()
    End Sub
#End Region

#Region " Main entry "
    '''===================================================================================================
    ''' <summary>
    ''' Main entry point.
    ''' </summary>
    '''===================================================================================================
    <STAThread()> _
    Public Shared Sub Main(ByVal args() As String)
        ' no args, if the exe is a SCR, run a screensaver.
        ' otherwise start as app.
        If args.Length <= 0 Then
            _RunningAsScreensaver = Application.ExecutablePath.ToUpper().EndsWith(".SCR")
            If RunningAsScreensaver Then
                ShowScreenSaver()
            Else
                ShowDesktopApp()
            End If
        Else
            ' we have args, determine what todo.
            Dim str As String = args(0).ToLower().Trim().Substring(0, 2)
            Select Case str
                Case "/c"
                    ' start in config mode.
                    ' read in full quote database.
                    ReadAllData(True)
                    Dim form As New ConfigForm()
                    form.ShowDialog()
                    Return

                Case "/p"
                    Return

                Case "/d"
                    ' start as desktop app.
                    ShowDesktopApp()
                    Return

                Case "/s"
                    ' start as screensaver app.
                    ShowScreenSaver()
                    Return
            End Select
            MessageBox.Show("Invalid command line argument :" & str, "Invalid Command Line Argument", MessageBoxButtons.OK, MessageBoxIcon.Hand)
        End If
    End Sub
#End Region

#Region " Desktop Mode "
    '''===================================================================================================
    ''' <summary>
    ''' Desktop mode.
    ''' </summary>
    '''===================================================================================================
    Private Shared Sub ShowDesktopApp()
        ' read in only selected quotes.
        ReadAllData(False)
        _RunningAsScreensaver = False

        ' determine what screen to use.
        Dim homeScreen As Screen = Nothing
        If Screen.AllScreens.Length > 1 Then
            Dim form As New ScreenSelectorForm()
            If form.ShowDialog() = DialogResult.OK Then
                homeScreen = form.SelectedScreen
            Else
                Return
            End If
        Else
            homeScreen = Screen.PrimaryScreen
        End If

        ' start the app on specified screen.
        If homeScreen IsNot Nothing Then
            Dim source As New PhotoQueue()
            Dim mainForm As New ScreenSaverForm(homeScreen, source)
            mainForm.Init()
            Application.Run(mainForm)
        End If
    End Sub
#End Region

#Region " Screensaver Mode "
    '''===================================================================================================
    ''' <summary>
    ''' Screensaver mode.
    ''' </summary>
    '''===================================================================================================
    Private Shared Sub ShowScreenSaver()
        ' read in only selected quotes.
        ReadAllData(False)

        ' start a form up on each screen.
        _RunningAsScreensaver = True
        Dim source As New PhotoQueue()
        For Each screen As Screen In screen.AllScreens
            Dim form As New ScreenSaverForm(screen, source)
            form.Init()
            form.Show()
        Next screen
        Application.Run()
    End Sub
#End Region

#Region " Read Data "
    '''===================================================================================================
    ''' <summary>
    ''' Main function to read quote data in.
    ''' </summary>
    '''===================================================================================================
    Public Shared Sub ReadAllData(ByVal ForceReadAll As Boolean)
        If ForceReadAll Then
            ReadBuiltInQuoteData()
            ReadCustomQuoteData()
            ReadWordLibrary()
            ReadEvents()
        Else
            Select Case My.Settings.QuoteFormat.ToLower()
                Case "none"
                    Return
                Case "all quotes"
                    ReadBuiltInQuoteData()
                    ReadCustomQuoteData()
                Case "built-in quotes"
                    ReadBuiltInQuoteData()
                Case "custom quotes"
                    ReadCustomQuoteData()
                Case "events or words only"
                    ' dont read in quotes at all.
            End Select
            Select Case My.Settings.AdditionalQuotes.ToLower()
                Case "none"
                Case "all"
                    ReadWordLibrary()
                    ReadEvents()
                Case "word definitions"
                    ReadWordLibrary()
                Case "world events"
                    ReadEvents()
            End Select
        End If
        '
        ShuffleArray(Words)
        ShuffleArray(Quotes)
        ConfigureCeilings()
    End Sub
    '''===================================================================================================
    ''' <summary>
    ''' Reads in custom quote data.
    ''' </summary>
    '''===================================================================================================
    Public Shared Sub ReadCustomQuoteData()
        If My.Settings.CustomQuotes Is Nothing Then
            Return
        End If
        For Each quoteLine As String In My.Settings.CustomQuotes
            Dim lineArray() As String
            Dim myQuote As New DisplayItem()
            lineArray = quoteLine.Split(New Char() {"|"c})
            '
            myQuote.TopText = lineArray(0)
            myQuote.TopText = myQuote.TopText.Replace("ý", Environment.NewLine)
            myQuote.TopText = myQuote.TopText.Replace(">", Environment.NewLine)
            If lineArray.Length >= 2 Then
                myQuote.BottomText = lineArray(1)
            End If
            '
            If myQuote.TopText.StartsWith("""") = False Then
                myQuote.TopText = """" & myQuote.TopText
            End If
            If myQuote.TopText.EndsWith("""") = False Then
                myQuote.TopText = myQuote.TopText & """"
            End If
            myQuote.TopText = myQuote.TopText.Replace("""""", """")
            myQuote.BottomText = myQuote.BottomText.Replace("""""", """")
            '
            Program.CustomQuotes.Add(myQuote)
        Next quoteLine
    End Sub
    '''===================================================================================================
    ''' <summary>
    ''' Reads in Word library.
    ''' </summary>
    '''===================================================================================================
    Public Shared Sub ReadWordLibrary()
        Dim buffer() As String
        buffer = My.Resources.Words.Split(New String() {Constants.vbLf, Constants.vbCrLf}, StringSplitOptions.RemoveEmptyEntries)
        For Each line As String In buffer
            Dim lineArray() As String
            Dim myWord As New DisplayItem()
            lineArray = line.Split(New Char() {"|"c})
            '
            myWord.BottomText = lineArray(2)
            myWord.TopText = lineArray(0)
            myWord.TopText = myWord.TopText.Replace("ý", Environment.NewLine)
            '
            If myWord.TopText.StartsWith("""") = False Then
                myWord.TopText = """" & myWord.TopText
            End If
            If myWord.TopText.EndsWith("""") = False Then
                myWord.TopText = myWord.TopText & """"
            End If
            myWord.TopText = String.Format("word of the day:{0}{1}", Environment.NewLine, myWord.TopText)
            myWord.TopText = myWord.TopText.Replace("""""", """")
            '
            Program.Words.Add(myWord)
        Next line
    End Sub
    '''===================================================================================================
    ''' <summary>
    ''' Reads in event data.
    ''' </summary>
    '''===================================================================================================
    Public Shared Sub ReadEvents()
        Dim buffer() As String
        buffer = My.Resources.Events.Split(New String() {Constants.vbLf, Constants.vbCrLf}, StringSplitOptions.RemoveEmptyEntries)
        For Each line As String In buffer
            Dim lineArray() As String
            Dim myEvent As New DisplayItem()
            lineArray = line.Split(New Char() {ControlChars.Tab})
            '
            Dim day As Integer
            Integer.TryParse(lineArray(0), day)
            Dim month As Integer
            Integer.TryParse(lineArray(1), month)
            Dim year As Integer
            Integer.TryParse(lineArray(2), year)
            '
            myEvent.Day = day
            myEvent.Month = month
            myEvent.Year = year
            '
            myEvent.BottomText = lineArray(3)
            myEvent.BottomText = myEvent.BottomText.Replace("ý", Environment.NewLine)
            '
            If myEvent.BottomText.StartsWith("""") = False Then
                myEvent.BottomText = """" & myEvent.BottomText
            End If
            If myEvent.BottomText.EndsWith("""") = False Then
                myEvent.BottomText = myEvent.BottomText & """"
            End If
            Dim yearText As String
            If year < 0 Then
                year = Math.Abs(year)
                yearText = String.Format("{0} BC", year)
            Else
                yearText = year.ToString()
            End If
            myEvent.BottomText = myEvent.BottomText.Replace("""""", """")
            myEvent.BottomText = String.Format("{1} [in the year {0}]", yearText, myEvent.BottomText)
            myEvent.TopText = "on this day..."
            '
            Program.Events.Add(myEvent)
        Next line
    End Sub
    '''===================================================================================================
    ''' <summary>
    ''' Reads in built in quote data.
    ''' </summary>
    '''===================================================================================================
    Public Shared Sub ReadBuiltInQuoteData()
        Dim buffer() As String
        buffer = My.Resources.Quotes.Split(New String() {Constants.vbLf, Constants.vbCrLf}, StringSplitOptions.RemoveEmptyEntries)
        For Each line As String In buffer
            Dim lineArray() As String
            Dim myQuote As New DisplayItem()
            lineArray = line.Split(New Char() {ControlChars.Tab})
            '
            myQuote.TopText = lineArray(0)
            myQuote.TopText = myQuote.TopText.Replace("ý", Environment.NewLine)
            myQuote.BottomText = lineArray(1)
            '
            If myQuote.TopText.StartsWith("""") = False Then
                myQuote.TopText = """" & myQuote.TopText
            End If
            If myQuote.TopText.EndsWith("""") = False Then
                myQuote.TopText = myQuote.TopText & """"
            End If
            myQuote.TopText = myQuote.TopText.Replace("""""", """")
            myQuote.BottomText = myQuote.BottomText.Replace("""""", """")
            '
            Program.Quotes.Add(myQuote)
        Next line
    End Sub
#End Region

#Region " Array Shuffle "
    '''===================================================================================================
    ''' <summary>
    ''' Shuffles the specified array.
    ''' </summary>
    ''' <param name="MyArray">The array to shuffle.</param>
    '''===================================================================================================
    Private Shared Sub ShuffleArray(ByVal MyArray As List(Of DisplayItem))
        SyncLock MyArray
            Dim num As Integer = 0
            For i As Integer = 0 To (MyArray.Count - 1) - 1
                Do While num = i
                    num = _MyRandom.Next(0, MyArray.Count)
                Loop
                Dim info As DisplayItem = MyArray(i)
                MyArray(i) = MyArray(num)
                MyArray(num) = info
                num = i + 1
            Next i
        End SyncLock
    End Sub
#End Region

#Region " Ceilings "
    '''===================================================================================================
    ''' <summary>
    ''' Configures the ceilings to display only a certain amount of data.
    ''' </summary>
    '''===================================================================================================
    Public Shared Sub ConfigureCeilings()
        If My.Settings.PercentQuotes <> 100 Then
            Dim percent As Double = My.Settings.PercentQuotes * 0.01
            _QuoteCeiling = CInt(Fix(Program.Quotes.Count * percent))
        Else
            _QuoteCeiling = Program.Quotes.Count
        End If
        '
        If My.Settings.PercentWords <> 100 Then
            Dim percent As Double = My.Settings.PercentWords * 0.01
            _WordsCeiling = CInt(Fix(Program.Words.Count * percent))
        Else
            _WordsCeiling = Program.Words.Count
        End If
    End Sub
#End Region

End Class